from django.contrib import admin


class DataImportableAdmin(admin.ModelAdmin):
    change_list_template = 'django_io/admin/change_list.html'
    upload_url = None

    def get_upload_url(self):
        return self.upload_url

    def changelist_view(self, request, extra_context=None):
        extra_context = {
            'upload_url': self.get_upload_url(),
            **(extra_context or {}),
        }
        return super(DataImportableAdmin, self).changelist_view(request, extra_context)


__all__ = [
    'DataImportableAdmin',
]



import codecs
import csv
from collections import OrderedDict

from django.db import transaction
from django.forms.utils import ErrorDict


class DataImporterMeta(type):
    """Collect Fields declared on the base classes."""

    @property
    def _label_to_keys_map(cls):
        return {
            field.label or key: key
            for key, field in cls.form_class.base_fields.items()
        }

    @property
    def _keys_to_labels_map(cls):
        return {
            key: field.label or key
            for key, field in cls.form_class.base_fields.items()
        }


class DataImporter(metaclass=DataImporterMeta):
    form_class = None

    @classmethod
    def check_extra_headers(cls, headers):
        return set(headers) - set(cls._label_to_keys_map.keys())

    @staticmethod
    def load_data_from_file(file):
        return csv.DictReader(codecs.iterdecode(file, 'utf-8-sig'))

    def translate_data(self, data):
        label_to_keys_map = self.__class__._label_to_keys_map
        return [
            {
                label_to_keys_map[label]: value
                for label, value in row.items()
                if label_to_keys_map.get(label)
            }
            for row in data
        ]

    def __init__(self, data):
        self.data = self.translate_data(data)
        self._errors = None
        self._forms = []

    def full_clean(self):
        self._errors = OrderedDict()
        for i, row in enumerate(self.data):
            form = self.form_class(row)
            self._forms.append(form)
            if not form.is_valid():
                self._errors[i] = form.errors

    @property
    def errors(self):
        if self._errors is None:
            self.full_clean()
        return self._errors

    @property
    def errors_formatted(self):
        keys_to_labels_map = self.__class__._keys_to_labels_map
        errors_formatted = OrderedDict()
        for i, error_row in self.errors.items():
            errors_formatted[i + 2] = ErrorDict(**{
                keys_to_labels_map[key]: errors
                for key, errors in error_row.items()
            })
        return errors_formatted

    def is_valid(self):
        return not self.errors

    def save(self):  # TODO: commit
        if self.errors:
            raise ValueError("Import could not be run because the data didn't validate")

        num_rows_written = 0
        with transaction.atomic():
            for form in self._forms:
                form.save()
                num_rows_written += 1
        return num_rows_written


__all__ = [
    'DataImporter',
]



from django import forms

__all__ = [
    'CurrencyField',
    'PercentageField',
    'ListField',
    'FormattedIntegerField',
]


class CurrencyField(forms.DecimalField):
    def to_python(self, value):
        value = str(value).strip()
        value = value.replace(',', '').lstrip('$')
        return super(CurrencyField, self).to_python(value)


class PercentageField(forms.DecimalField):
    def to_python(self, value):
        if value in self.empty_values:
            return None
        value = str(value).strip()
        if value[-1] == '%':
            value = super(PercentageField, self).to_python(value[:-1]) / 100
        else:
            value = super(PercentageField, self).to_python(value)
        return value


class ListField(forms.CharField):
    def __init__(self, seperator=',', *args, **kwargs):
        self.seperator = seperator
        super(ListField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        value = super(ListField, self).to_python(value)
        return [item.strip() for item in value.split(self.seperator)]


class FormattedIntegerField(forms.IntegerField):
    def to_python(self, value):
        value = value.replace(',', '')
        return super(FormattedIntegerField, self).to_python(value)
